@@ -18,8 +18,6 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h> // For POSTGRES_UTILS_PATH
-
 #include <libglom/connectionpool_backends/postgres_self.h>
 #include <libglom/utils.h>
 #include <libglom/spawn_with_feedback.h>
@@ -30,13 +28,14 @@
 
 #include <libglom/gst-package.h>
 #include <sstream> //For stringstream
+#include <iostream>
 
 #ifdef G_OS_WIN32
 # include <windows.h>
 # include <winsock2.h>
 #else
 # include <sys/types.h>
-# include <sys/socket.h> 
+# include <sys/socket.h>
 # include <errno.h>
 # include <netinet/in.h> //For sockaddr_in
 #endif
@@ -46,22 +45,6 @@
 // Uncomment to see debug messages
 //#define GLOM_CONNECTION_DEBUG
 
-namespace
-{
-
-static Glib::ustring port_as_string(int port_num)
-{
-  Glib::ustring result;
-  char* cresult = g_strdup_printf("%d", port_num);
-  if(cresult)
-    result = cresult;
-  g_free(cresult);
-
-  return result;
-}
-
-} // anonymous namespace
-
 namespace Glom
 {
 
@@ -70,9 +53,9 @@ namespace ConnectionPoolBackends
 
 //TODO: Do we need these sameuser lines?
 
-// We need both <=8.3 and >=8.4 versions, because the ident line changed syntax 
+// We need both <=8.3 and >=8.4 versions, because the ident line changed syntax
 // incompatibly: http://www.postgresql.org/about/press/features84#security
- 
+
 #define DEFAULT_CONFIG_PG_HBA_LOCAL_8p3 \
 "# TYPE  DATABASE    USER        CIDR-ADDRESS          METHOD\n" \
 "\n" \
@@ -123,50 +106,14 @@ DEFAULT_CONFIG_PG_HBA_REMOTE_EXTRA
 #define PORT_POSTGRESQL_SELF_HOSTED_START 5433
 #define PORT_POSTGRESQL_SELF_HOSTED_END 5500
 
-#define DEFAULT_CONFIG_PG_IDENT ""
+static const char* DEFAULT_CONFIG_PG_IDENT = "";
+static const char* FILENAME_DATA = "data";
+static const char* FILENAME_BACKUP = "backup";
 
 PostgresSelfHosted::PostgresSelfHosted()
-: m_port(0),
-   m_network_shared(false)
+: m_network_shared(false)
 {
-}
-
-
-std::string PostgresSelfHosted::get_path_to_postgres_executable(const std::string& program)
-{
-#ifdef G_OS_WIN32
-  // Add the .exe extension on Windows:
-  std::string real_program = program + EXEEXT;
-    
-  // Have a look at the bin directory of the application executable first.
-  // The installer installs postgres there. postgres needs to be installed
-  // in a directory called bin for its relocation stuff to work, so that
-  // it finds the share data in share. Unfortunately it does not look into
-  // share/postgresql which would be nice to separate the postgres stuff
-  // from the other shared data. We can perhaps still change this later by
-  // building postgres with another prefix than /local/pgsql.
-  gchar* installation_directory = g_win32_get_package_installation_directory_of_module(0);
-  std::string test = Glib::build_filename(installation_directory, Glib::build_filename("bin", real_program));
-  g_free(installation_directory);
-
-  if(Glib::file_test(test, Glib::FILE_TEST_IS_EXECUTABLE))
-    return test;
-
-  // Look in PATH otherwise
-  return Glib::find_program_in_path(real_program);
-#else // G_OS_WIN32
-  return Glib::build_filename(POSTGRES_UTILS_PATH, program + EXEEXT);
-#endif // !G_OS_WIN32
-}
-
-void PostgresSelfHosted::set_self_hosting_data_uri(const std::string& data_uri)
-{
-  if(m_self_hosting_data_uri != data_uri)
-  {
-    // Can't change data uri if we are running the server on another data uri
-    g_assert(!get_self_hosting_active());
-    m_self_hosting_data_uri = data_uri;
-  }
+  m_host = "localhost";
 }
 
 bool PostgresSelfHosted::get_self_hosting_active() const
@@ -174,7 +121,7 @@ bool PostgresSelfHosted::get_self_hosting_active() const
   return m_port != 0;
 }
 
-int PostgresSelfHosted::get_port() const
+unsigned int PostgresSelfHosted::get_port() const
 {
   return m_port;
 }
@@ -194,14 +141,14 @@ bool PostgresSelfHosted::install_postgres(const SlotProgress& /* slot_progress *
 
   //Careful. Maybe you want a different version.
   //Also, Glom will start its own instance of PostgreSQL, on its own port, when it needs to,
-  //so there is no need to start a Glom service after installation at system startup, 
+  //so there is no need to start a Glom service after installation at system startup,
   //though it will not hurt Glom if you do that.
   const gchar *packages[] = { "postgresql-8.1", 0 };
   const bool result = gst_packages_install(parent_window->gobj() /* parent window */, packages);
   if(result)
   {
     std::cout << "Glom: gst_packages_install() reports success." << std::endl;
-    //Double-check, because gst_packages_install() incorrectly returns TRUE if it fails because 
+    //Double-check, because gst_packages_install() incorrectly returns TRUE if it fails because
     //a) synaptic is already running, or
     //b) synaptic did not know about the package (no warning is shown in this case.)
     //Maybe gst_packages_install() never returns FALSE.
@@ -221,12 +168,12 @@ Backend::InitErrors PostgresSelfHosted::initialize(const SlotProgress& slot_prog
 {
   m_network_shared = network_shared;
 
-  if(m_self_hosting_data_uri.empty())
+  if(m_database_directory_uri.empty())
   {
-    std::cerr << "PostgresSelfHosted::initialize: m_self_hosting_data_uri is empty." << std::endl;
+    std::cerr << "PostgresSelfHosted::initialize: m_database_directory_uri is empty." << std::endl;
     return INITERROR_OTHER;
   }
-  
+
   if(initial_username.empty())
   {
     std::cerr << "PostgresSelfHosted::initialize(). Username was empty while attempting to create self-hosting database" << std::endl;
@@ -234,34 +181,29 @@ Backend::InitErrors PostgresSelfHosted::initialize(const SlotProgress& slot_prog
   }
 
   //Get the filepath of the directory that we should create:
-  const std::string dbdir_uri = m_self_hosting_data_uri;
+  const std::string dbdir_uri = m_database_directory_uri;
   //std::cout << "debug: dbdir_uri=" << dbdir_uri << std::endl;
 
-  if(directory_exists_uri(dbdir_uri))
+  if(file_exists_uri(dbdir_uri))
     return INITERROR_DIRECTORY_ALREADY_EXISTS;
 
   const std::string dbdir = Glib::filename_from_uri(dbdir_uri);
   //std::cout << "debug: dbdir=" << dbdir << std::endl;
   g_assert(!dbdir.empty());
 
-  //0770 means "this user and his group can read and write and "execute" (meaning add sub-files) this non-executable file".
-  //The 0 prefix means that this is octal.
-  int mkdir_succeeded = g_mkdir_with_parents(dbdir.c_str(), 0770);
-  if(mkdir_succeeded == -1)
+  const bool dbdir_created = create_directory_filepath(dbdir);
+  if(!dbdir_created)
   {
-    std::cerr << "Error from g_mkdir_with_parents() while trying to create directory: " << dbdir << std::endl;
-    perror("Error from g_mkdir_with_parents");
-    
+    std::cerr << "Couldn't create directory: " << dbdir << std::endl;
+
     return INITERROR_COULD_NOT_CREATE_DIRECTORY;
   }
 
   //Create the config directory:
-  const std::string dbdir_config = dbdir + "/config";
-  mkdir_succeeded = g_mkdir_with_parents(dbdir_config.c_str(), 0770);
-  if(mkdir_succeeded == -1)
+  const std::string dbdir_config = get_self_hosting_config_path(true /* create */);
+  if(dbdir_config.empty())
   {
-    std::cerr << "Error from g_mkdir_with_parents() while trying to create directory: " << dbdir_config << std::endl;
-    perror("Error from g_mkdir_with_parents");
+    std::cerr << "Couldn't create the config directory: " << dbdir << std::endl;
 
     return INITERROR_COULD_NOT_CREATE_DIRECTORY;
   }
@@ -270,11 +212,14 @@ Backend::InitErrors PostgresSelfHosted::initialize(const SlotProgress& slot_prog
   set_network_shared(slot_progress, m_network_shared); //Creates pg_hba.conf and pg_ident.conf
 
   //Check that there is not an existing data directory:
-  const std::string dbdir_data = dbdir + "/data";
-  mkdir_succeeded = g_mkdir_with_parents(dbdir_data.c_str(), 0770);
-  g_assert(mkdir_succeeded != -1);
+  const std::string dbdir_data = get_self_hosting_data_path(true /* create */);
+  if(dbdir_data.empty())
+  {
+    std::cerr << "Couldn't create the data directory: " << dbdir << std::endl;
+
+    return INITERROR_COULD_NOT_CREATE_DIRECTORY;
+  }
 
-  
   // initdb creates a new postgres database cluster:
 
   //Get file:// URI for the tmp/ directory:
@@ -298,7 +243,7 @@ Backend::InitErrors PostgresSelfHosted::initialize(const SlotProgress& slot_prog
 
   const int temp_pwfile_removed = g_remove(temp_pwfile.c_str()); //Of course, we don't want this to stay around. It would be a security risk.
   g_assert(temp_pwfile_removed == 0);
- 
+
   return result ? INITERROR_NONE : INITERROR_COULD_NOT_START_SERVER;
 }
 
@@ -320,8 +265,8 @@ Glib::ustring PostgresSelfHosted::get_postgresql_utils_version(const SlotProgres
   //Use a regex to get the version number:
   Glib::RefPtr<Glib::Regex> regex;
 
-  //We want the characters at the end:  
-  const gchar* VERSION_REGEX = "pg_ctl \\(PostgreSQL\\) (.*)";
+  //We want the characters at the end:
+  const gchar VERSION_REGEX[] = "pg_ctl \\(PostgreSQL\\) (.*)";
 
   #ifdef GLIBMM_EXCEPTIONS_ENABLED
   try
@@ -332,7 +277,7 @@ Glib::ustring PostgresSelfHosted::get_postgresql_utils_version(const SlotProgres
   {
     std::cerr << "Glom: Glib::Regex::create() failed: " << ex.what() << std::endl;
     return result;
-  } 
+  }
   #else
   std::auto_ptr<Glib::Error> ex;
   regex = Glib::Regex::create(VERSION_REGEX, static_cast<Glib::RegexCompileFlags>(0), static_cast<Glib::RegexMatchFlags>(0), ex);
@@ -342,7 +287,7 @@ Glib::ustring PostgresSelfHosted::get_postgresql_utils_version(const SlotProgres
     return result;
   }
   #endif
- 
+
   if(!regex)
     return result;
 
@@ -360,7 +305,7 @@ Glib::ustring PostgresSelfHosted::get_postgresql_utils_version(const SlotProgres
     if(!str.empty())
       return str; //Found.
   }
- 
+
   return result;
 }
 
@@ -372,8 +317,8 @@ float PostgresSelfHosted::get_postgresql_utils_version_as_number(const SlotProgr
 
   Glib::RefPtr<Glib::Regex> regex;
 
-  //We want the characters at the end:  
-  const gchar* VERSION_REGEX = "^(\\d*)\\.(\\d*)";
+  //We want the characters at the end:
+  const gchar VERSION_REGEX[] = "^(\\d*)\\.(\\d*)";
 
   #ifdef GLIBMM_EXCEPTIONS_ENABLED
   try
@@ -384,7 +329,7 @@ float PostgresSelfHosted::get_postgresql_utils_version_as_number(const SlotProgr
   {
     std::cerr << "Glom: Glib::Regex::create() failed: " << ex.what() << std::endl;
     return result;
-  } 
+  }
   #else
   std::auto_ptr<Glib::Error> ex;
   regex = Glib::Regex::create(VERSION_REGEX, static_cast<Glib::RegexCompileFlags>(0), static_cast<Glib::RegexMatchFlags>(0), ex);
@@ -394,7 +339,7 @@ float PostgresSelfHosted::get_postgresql_utils_version_as_number(const SlotProgr
     return result;
   }
   #endif
- 
+
   if(!regex)
     return result;
 
@@ -431,7 +376,7 @@ float PostgresSelfHosted::get_postgresql_utils_version_as_number(const SlotProgr
 }
 
 
-bool PostgresSelfHosted::startup(const SlotProgress& slot_progress, bool network_shared)
+Backend::StartupErrors PostgresSelfHosted::startup(const SlotProgress& slot_progress, bool network_shared)
 {
   m_network_shared = network_shared;
 
@@ -440,40 +385,51 @@ bool PostgresSelfHosted::startup(const SlotProgress& slot_progress, bool network
   //g_assert(!get_self_hosting_active());
 
   if(get_self_hosting_active())
-    return true; //Just do it once.
-
-  const std::string dbdir_uri = m_self_hosting_data_uri;
-
-  if(!(directory_exists_uri(dbdir_uri)))
   {
-    //TODO: Use a return enum or exception so we can tell the user about this:
-    std::cerr << "ConnectionPool::create_self_hosting(): The data directory could not be found: " << dbdir_uri << std::endl;
-    return false;
+    std::cerr << G_STRFUNC << ": Already started." << std::endl;
+    return STARTUPERROR_NONE; //Just do it once.
+  }
+
+  const std::string dbdir_uri = m_database_directory_uri;
+
+  if(!(file_exists_uri(dbdir_uri)))
+  {
+    std::cerr << "ConnectionPool::create_self_hosting(): The directory could not be found: " << dbdir_uri << std::endl;
+    return STARTUPERROR_FAILED_UNKNOWN_REASON;
+  }
+
+  const std::string dbdir = Glib::filename_from_uri(dbdir_uri);
+  g_assert(!dbdir.empty());
+
+  const std::string dbdir_data = Glib::build_filename(dbdir, FILENAME_DATA);
+  const Glib::ustring dbdir_data_uri = Glib::filename_to_uri(dbdir_data);
+  if(!(file_exists_uri(dbdir_data_uri)))
+  {
+    const std::string dbdir_backup = Glib::build_filename(dbdir, FILENAME_BACKUP);
+    const Glib::ustring dbdir_backup_uri = Glib::filename_to_uri(dbdir_backup);
+    if(file_exists_uri(dbdir_backup_uri))
+    {
+      std::cerr << G_STRFUNC << ": There is no data, but there is backup data." << std::endl;
+      //Let the caller convert the backup to real data and then try again:
+      return STARTUPERROR_FAILED_NO_DATA_HAS_BACKUP_DATA;
+    }
+    else
+    {
+      std::cerr << "ConnectionPool::create_self_hosting(): The data sub-directory could not be found." << dbdir_data_uri << std::endl;
+      return STARTUPERROR_FAILED_NO_DATA;
+    }
   }
 
   //Attempt to ensure that the config files are correct:
   set_network_shared(slot_progress, m_network_shared); //Creates pg_hba.conf and pg_ident.conf
 
-  const std::string dbdir = Glib::filename_from_uri(dbdir_uri);
-  g_assert(!dbdir.empty());
 
-  const std::string dbdir_data = Glib::build_filename(dbdir, "data");
-  const std::string dbdir_data_uri = Glib::filename_to_uri(dbdir_data);
-  if(!(directory_exists_uri(dbdir_data_uri)))
-  {
-    //TODO: Use a return enum or exception so we can tell the user about this:
-    std::cerr << "ConnectionPool::create_self_hosting(): The data sub-directory could not be found." << dbdir_data_uri << std::endl;
-    return false;
-  }
-
-
-  const int available_port = discover_first_free_port(PORT_POSTGRESQL_SELF_HOSTED_START, PORT_POSTGRESQL_SELF_HOSTED_END);
+  const unsigned int available_port = discover_first_free_port(PORT_POSTGRESQL_SELF_HOSTED_START, PORT_POSTGRESQL_SELF_HOSTED_END);
   //std::cout << "ConnectionPool::create_self_hosting():() : debug: Available port for self-hosting: " << available_port << std::endl;
   if(available_port == 0)
   {
-    //TODO: Use a return enum or exception so we can tell the user about this:
     std::cerr << "ConnectionPool::create_self_hosting(): No port was available between " << PORT_POSTGRESQL_SELF_HOSTED_START << " and " << PORT_POSTGRESQL_SELF_HOSTED_END << std::endl;
-    return false;
+    return STARTUPERROR_FAILED_UNKNOWN_REASON;
   }
 
   //TODO: Performance:
@@ -482,7 +438,6 @@ bool PostgresSelfHosted::startup(const SlotProgress& slot_progress, bool network
   // -D specifies the data directory.
   // -c config_file= specifies the configuration file
   // -k specifies a directory to use for the socket. This must be writable by us.
-  // POSTGRES_UTILS_PATH is defined in config.h, based on the configure.
   // Make sure to use double quotes for the executable path, because the
   // CreateProcess() API used on Windows does not support single quotes.
   const std::string command_postgres_start = "\"" + get_path_to_postgres_executable("postgres") + "\" -D \"" + dbdir_data + "\" "
@@ -502,7 +457,7 @@ bool PostgresSelfHosted::startup(const SlotProgress& slot_progress, bool network
   //This is a big hack that we should avoid. murrayc.
   //
   //pg_ctl actually seems to return a 0 result code for "is running" and a 1 for not running, at least with Postgres 8.2,
-  //so maybe we can avoid this in future.  
+  //so maybe we can avoid this in future.
   //Please do test it with your postgres version, using "echo $?" to see the result code of the last command.
   const std::string second_command_success_text = "is running"; //TODO: This is not a stable API. Also, watch out for localisation.
 
@@ -511,37 +466,36 @@ bool PostgresSelfHosted::startup(const SlotProgress& slot_progress, bool network
   if(!result)
   {
     std::cerr << "Error while attempting to self-host a database." << std::endl;
-    return false;
+    return STARTUPERROR_FAILED_UNKNOWN_REASON;
   }
 
   m_port = available_port; //Remember it for later.
-  
-  return true;
+
+  return STARTUPERROR_NONE;
 }
 
-void PostgresSelfHosted::cleanup(const SlotProgress& slot_progress)
+bool PostgresSelfHosted::cleanup(const SlotProgress& slot_progress)
 {
   // This seems to be called twice sometimes, so we don't assert here until
   // this is fixed.
   //g_assert(get_self_hosting_active());
 
   if(!get_self_hosting_active())
-    return; //Don't try to stop it if we have not started it.
+    return true; //Don't try to stop it if we have not started it.
 
-  const std::string dbdir_uri = m_self_hosting_data_uri;
+  const std::string dbdir_uri = m_database_directory_uri;
   const std::string dbdir = Glib::filename_from_uri(dbdir_uri);
   g_assert(!dbdir.empty());
 
-  const std::string dbdir_data = Glib::build_filename(dbdir, "data");
+  const std::string dbdir_data = Glib::build_filename(dbdir, FILENAME_DATA);
 
 
-  // TODO: Detect other instances on the same computer, and use a different port number, 
+  // TODO: Detect other instances on the same computer, and use a different port number,
   // or refuse to continue, showing an error dialog.
 
   // -D specifies the data directory.
   // -c config_file= specifies the configuration file
   // -k specifies a directory to use for the socket. This must be writable by us.
-  // POSTGRES_UTILS_PATH is defined in config.h, based on the configure.
   // We use "-m fast" instead of the default "-m smart" because that waits for clients to disconnect (and sometimes never succeeds).
   // TODO: Warn about connected clients on other computers? Warn those other users?
   // Make sure to use double quotes for the executable path, because the
@@ -558,10 +512,13 @@ void PostgresSelfHosted::cleanup(const SlotProgress& slot_progress)
     if(!result)
     {
       std::cerr << "Error while attempting (for a second time) to stop self-hosting of the database."  << std::endl;
+      return false;
     }
   }
 
   m_port = 0;
+
+  return true;
 }
 
 
@@ -572,13 +529,13 @@ bool PostgresSelfHosted::set_network_shared(const SlotProgress& slot_progress, b
 
   m_network_shared = network_shared;
 
-  const std::string dbdir_uri = m_self_hosting_data_uri;
+  const std::string dbdir_uri = m_database_directory_uri;
   const std::string dbdir = Glib::filename_from_uri(dbdir_uri);
 
   const std::string dbdir_uri_config = dbdir_uri + "/config";
   const char* default_conf_contents = 0;
 
-  // Choose the configuration contents based on the postgresql version 
+  // Choose the configuration contents based on the postgresql version
   // and whether we want to be network-shared:
   const float postgresql_version = get_postgresql_utils_version_as_number(slot_progress);
   //std::cout << "DEBUG: postgresql_version=" << postgresql_version << std::endl;
@@ -606,7 +563,7 @@ static bool on_timeout_delay(const Glib::RefPtr<Glib::MainLoop>& mainloop)
   //Allow our mainloop.run() to return:
   if(mainloop)
     mainloop->quit();
-    
+
   return false;
 }
 
@@ -627,13 +584,13 @@ Glib::RefPtr<Gnome::Gda::Connection> PostgresSelfHosted::connect(const Glib::ust
   const guint MAX_RETRIES_KNOWN_PASSWORD = 30; /* seconds */
   const guint MAX_RETRIES_EVER = 60; /* seconds */
   while(keep_trying)
-  { 
-    result = attempt_connect("localhost", port_as_string(m_port), database, username, password, ex);
-    if(!result && 
+  {
+    result = attempt_connect(port_as_string(m_port), database, username, password, ex);
+    if(!result &&
       ex.get() && (ex->get_failure_type() == ExceptionConnection::FAILURE_NO_SERVER))
     {
       //It must be using a default password, so any failure would not be due to a wrong password.
-      //However, pg_ctl sometimes reports success before it is really ready to let us connect, 
+      //However, pg_ctl sometimes reports success before it is really ready to let us connect,
       //so in this case we can just keep trying until it works, with a very long timeout.
       count_retries++;
       const guint max_retries = m_network_shared ? MAX_RETRIES_EVER : MAX_RETRIES_KNOWN_PASSWORD;
@@ -643,20 +600,20 @@ Glib::RefPtr<Gnome::Gda::Connection> PostgresSelfHosted::connect(const Glib::ust
         continue;
       }
 
-      std::cout << "DEBUG: Glom::PostgresSelfHosted::connect(): Waiting and retrying the connection due to suspected too-early success of pg_ctl." << std::endl; 
+      std::cout << "DEBUG: Glom::PostgresSelfHosted::connect(): Waiting and retrying the connection due to suspected too-early success of pg_ctl." << std::endl;
 
       //Wait:
       Glib::RefPtr<Glib::MainLoop> mainloop = Glib::MainLoop::create(false);
         sigc::connection connection_timeout = Glib::signal_timeout().connect(
-        sigc::bind(sigc::ptr_fun(&on_timeout_delay), sigc::ref(mainloop)), 
+        sigc::bind(sigc::ptr_fun(&on_timeout_delay), sigc::ref(mainloop)),
         1000 /* 1 second */);
       mainloop->run();
       connection_timeout.disconnect();
-      
+
       keep_trying = true;
       continue;
     }
-    
+
     keep_trying = false;
   }
 
@@ -666,10 +623,10 @@ Glib::RefPtr<Gnome::Gda::Connection> PostgresSelfHosted::connect(const Glib::ust
 
 bool PostgresSelfHosted::create_database(const Glib::ustring& database_name, const Glib::ustring& username, const Glib::ustring& password, std::auto_ptr<Glib::Error>& error)
 {
-  return attempt_create_database(database_name, "localhost", port_as_string(m_port), username, password, error);
+  return attempt_create_database(database_name, m_host, port_as_string(m_port), username, password, error);
 }
 
-int PostgresSelfHosted::discover_first_free_port(int start_port, int end_port)
+unsigned int PostgresSelfHosted::discover_first_free_port(unsigned int start_port, unsigned int end_port)
 {
   //Open a socket so we can try to bind it to a port:
   const int fd = socket(AF_INET, SOCK_STREAM, 0);
@@ -689,7 +646,7 @@ int PostgresSelfHosted::discover_first_free_port(int start_port, int end_port)
   memset(&sa, 0, sizeof(sa));
   sa.sin_family = AF_INET;
 
-  int port_to_try = start_port;
+  guint16 port_to_try = start_port;
   while (port_to_try <= end_port)
   {
     sa.sin_port = htons(port_to_try);
@@ -705,7 +662,7 @@ int PostgresSelfHosted::discover_first_free_port(int start_port, int end_port)
       #endif // G_OS_WIN32
 
       //Some BSDs don't have this.
-      //But watch out - if you don't include errno.h then this won't be 
+      //But watch out - if you don't include errno.h then this won't be
       //defined on Linux either, but you really do need to check for it.
       #ifdef EADDRINUSE
       available = (errno != EADDRINUSE);
@@ -745,86 +702,6 @@ int PostgresSelfHosted::discover_first_free_port(int start_port, int end_port)
   return 0;
 }
 
-bool PostgresSelfHosted::create_text_file(const std::string& file_uri, const std::string& contents)
-{
-  Glib::RefPtr<Gio::File> file = Gio::File::create_for_uri(file_uri);
-  Glib::RefPtr<Gio::FileOutputStream> stream;
-
-  //Create the file if it does not already exist:
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
-  try
-  {
-    if(file->query_exists())
-    {
-      stream = file->replace(); //Instead of append_to().
-    }
-    else
-    {
-      //By default files created are generally readable by everyone, but if we pass FILE_CREATE_PRIVATE in flags the file will be made readable only to the current user, to the level that is supported on the target filesystem.
-      //TODO: Do we want to specify 0660 exactly? (means "this user and his group can read and write this non-executable file".)
-      stream = file->create_file();
-    }
-  }
-  catch(const Gio::Error& ex)
-  {
-#else
-  std::auto_ptr<Gio::Error> error;
-  stream.create(error);
-  if(error.get())
-  {
-    const Gio::Error& ex = *error.get();
-#endif
-    // If the operation was not successful, print the error and abort
-    std::cerr << "ConnectionPool::create_text_file(): exception while creating file." << std::endl
-      << "  file uri:" << file_uri << std::endl
-      << "  error:" << ex.what() << std::endl;
-    return false; // print_error(ex, output_uri_string);
-  }
-
-
-  if(!stream)
-    return false;
-
-
-  gsize bytes_written = 0;
-  const std::string::size_type contents_size = contents.size();
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
-  try
-  {
-    //Write the data to the output uri
-    bytes_written = stream->write(contents.data(), contents_size);
-  }
-  catch(const Gio::Error& ex)
-  {
-#else
-  bytes_written = stream->write(contents.data(), contents_size, error);
-  if(error.get())
-  {
-    Gio::Error& ex = *error.get();
-#endif
-    // If the operation was not successful, print the error and abort
-    std::cerr << "ConnectionPool::create_text_file(): exception while writing to file." << std::endl
-      << "  file uri:" << file_uri << std::endl
-      << "  error:" << ex.what() << std::endl;
-    return false; //print_error(ex, output_uri_string);
-  }
-
-  if(bytes_written != contents_size)
-  {
-    std::cerr << "ConnectionPool::create_text_file(): not all bytes written when writing to file." << std::endl
-      << "  file uri:" << file_uri << std::endl;
-    return false;
-  }
-
-  return true; //Success.
-}
-
-bool PostgresSelfHosted::directory_exists_uri(const std::string& uri)
-{
-  Glib::RefPtr<Gio::File> file = Gio::File::create_for_uri(uri);
-  return file && file->query_exists();
-}
-
 } // namespace ConnectionPoolBackends
 
 } // namespcae Glom
