@@ -136,11 +136,11 @@ Glib::ustring Utils::string_replace(const Glib::ustring& src, const Glib::ustrin
 {
   if(search_for.empty())
   {
-    std::cerr << "Utils::string_replace(): search_for was empty." << std::endl;
+    std::cerr << G_STRFUNC << ": search_for was empty." << std::endl;
     return src;
   }
 
-  //std::cout << "debug: Utils::string_replace(): src=" << src << ", search_for=" << search_for << ", replace_with=" << replace_with << std::endl;
+  //std::cout << "debug: " << G_STRFUNC << ": src=" << src << ", search_for=" << search_for << ", replace_with=" << replace_with << std::endl;
 
   std::string result = src;
 
@@ -195,7 +195,7 @@ Glib::ustring Utils::string_replace(const Glib::ustring& src, const Glib::ustrin
 }
 
 
-Glib::ustring Utils::build_sql_select_with_where_clause(const Glib::ustring& table_name, const type_vecLayoutFields& fieldsToGet, const Glib::ustring& where_clause, const Glib::ustring& extra_join, const type_sort_clause& sort_clause, const Glib::ustring& extra_group_by)
+Glib::RefPtr<Gnome::Gda::SqlBuilder> Utils::build_sql_select_with_where_clause(const Glib::ustring& table_name, const type_vecLayoutFields& fieldsToGet, const Gnome::Gda::SqlExpr& where_clause, const sharedptr<const Relationship>& extra_join, const type_sort_clause& sort_clause, guint limit)
 {
   //TODO_Performance:
   type_vecConstLayoutFields constFieldsToGet;
@@ -204,7 +204,7 @@ Glib::ustring Utils::build_sql_select_with_where_clause(const Glib::ustring& tab
     constFieldsToGet.push_back(*iter);
   }
 
-  return build_sql_select_with_where_clause(table_name, constFieldsToGet, where_clause, extra_join, sort_clause, extra_group_by);
+  return build_sql_select_with_where_clause(table_name, constFieldsToGet, where_clause, extra_join, sort_clause, limit);
 }
 
 
@@ -236,16 +236,62 @@ static void add_to_relationships_list(type_list_relationships& list_relationship
     list_relationships.push_back(uses_rel);
   }
 
- 
 }
 
-
-Glib::ustring Utils::build_sql_select_fields_to_get(const Glib::ustring& table_name, const type_vecConstLayoutFields& fieldsToGet, const type_sort_clause& sort_clause, Glib::ustring& sql_part_from, Glib::ustring& sql_part_leftouterjoin)
+static void builder_add_join(const Glib::RefPtr<Gnome::Gda::SqlBuilder>& builder, const sharedptr<const UsesRelationship>& uses_relationship)
 {
-  //Initialize output parameters:
-  sql_part_from = Glib::ustring();
-  sql_part_leftouterjoin = Glib::ustring();
+  sharedptr<const Relationship> relationship = uses_relationship->get_relationship();
+  if(!relationship->get_has_fields()) //TODO: Handle related_record has_fields.
+  {
+    if(relationship->get_has_to_table())
+    {
+      //It is a relationship that only specifies the table, without specifying linking fields:
+      builder->select_add_target(relationship->get_to_table());
+    }
 
+    return;
+  }
+
+  // Define the alias name as returned by get_sql_join_alias_name():
+
+  // Specify an alias, to avoid ambiguity when using 2 relationships to the same table.
+  const Glib::ustring alias_name = uses_relationship->get_sql_join_alias_name();
+
+  // Add the JOIN:
+  if(!uses_relationship->get_has_related_relationship_name())
+  {
+    const guint to_target_id = builder->select_add_target(relationship->get_to_table(), alias_name);
+
+    builder->select_join_targets(
+      builder->select_add_target(relationship->get_from_table()),
+      to_target_id,
+      Gnome::Gda::SQL_SELECT_JOIN_LEFT,
+      builder->add_cond(
+        Gnome::Gda::SQL_OPERATOR_TYPE_EQ,
+        builder->add_field_id(relationship->get_from_field(), relationship->get_from_table()),
+        builder->add_field_id(relationship->get_to_field(), alias_name)));
+  }
+  else
+  {
+     UsesRelationship parent_relationship;
+     parent_relationship.set_relationship(relationship);
+     sharedptr<const Relationship> related_relationship = uses_relationship->get_related_relationship();
+
+     const guint to_target_id = builder->select_add_target(related_relationship->get_to_table(), alias_name);
+
+     builder->select_join_targets(
+       builder->select_add_target(relationship->get_from_table()), //TODO: Must we use the ID from select_add_target_id()?
+       to_target_id,
+       Gnome::Gda::SQL_SELECT_JOIN_LEFT,
+       builder->add_cond(
+         Gnome::Gda::SQL_OPERATOR_TYPE_EQ,
+         builder->add_field_id(related_relationship->get_from_field(), parent_relationship.get_sql_join_alias_name()),
+         builder->add_field_id(related_relationship->get_to_field(), alias_name) ) );
+  }
+}
+
+void Utils::build_sql_select_add_fields_to_get(const Glib::RefPtr<Gnome::Gda::SqlBuilder>& builder, const Glib::ustring& table_name, const type_vecConstLayoutFields& fieldsToGet, const type_sort_clause& sort_clause, bool extra_join)
+{
   //Get all relationships used in the query:
   typedef std::list< sharedptr<const UsesRelationship> > type_list_relationships;
   type_list_relationships list_relationships;
@@ -263,8 +309,15 @@ Glib::ustring Utils::build_sql_select_fields_to_get(const Glib::ustring& table_n
   }
 
 
-  Glib::ustring sql_part_fields;
+  //LEFT OUTER JOIN will get the field values from the other tables,
+  //and give us our fields for this table even if there is no corresponding value in the other table.
+  for(type_list_relationships::const_iterator iter = list_relationships.begin(); iter != list_relationships.end(); ++iter)
+  {
+    sharedptr<const UsesRelationship> uses_relationship = *iter;
+    builder_add_join(builder, uses_relationship);
+  }
 
+  bool one_added = false;
   for(type_vecConstLayoutFields::const_iterator iter = fieldsToGet.begin(); iter != fieldsToGet.end(); ++iter)
   {
     Glib::ustring one_sql_part;
@@ -276,121 +329,105 @@ Glib::ustring Utils::build_sql_select_fields_to_get(const Glib::ustring& table_n
       continue;
     }
 
-    bool is_summary = false;
+    //Get the parent, such as the table name, or the alias name for the join:
+    const Glib::ustring parent = layout_item->get_sql_table_or_join_alias_name(table_name);
+
     const LayoutItem_FieldSummary* fieldsummary = dynamic_cast<const LayoutItem_FieldSummary*>(layout_item.obj());
     if(fieldsummary)
-      is_summary = true;
-
-    //Add, for instance, "SUM(":
-    if(is_summary)
-      one_sql_part += fieldsummary->get_summary_type_sql() + '(';
-
-    one_sql_part += layout_item->get_sql_name(table_name);
-
-    //Close the summary bracket if necessary.
-    if(is_summary)
-      one_sql_part +=  ')';
-
-    //Append it to the big string of fields:
-    if(!one_sql_part.empty())
     {
-      if(!sql_part_fields.empty())
-        sql_part_fields += ", ";
-
-      sql_part_fields += one_sql_part;
+      const Gnome::Gda::SqlBuilder::Id id_function = builder->add_function(
+        fieldsummary->get_summary_type_sql(),
+        builder->add_id(layout_item->get_sql_name(table_name)) ); //TODO: Just use add_field_id()?
+      builder->add_field_value_id(id_function);
     }
-  }
-
-  if(sql_part_fields.empty())
-  {
-    std::cerr << "Utils::build_sql_select_fields_to_get(): sql_part_fields.empty(): fieldsToGet.size()=" << fieldsToGet.size() << std::endl;
-    return sql_part_fields;
-  }
-
-  //LEFT OUTER JOIN will get the field values from the other tables, 
-  //and give us our fields for this table even if there is no corresponding value in the other table.
-  for(type_list_relationships::const_iterator iter = list_relationships.begin(); iter != list_relationships.end(); ++iter)
-  {
-    sharedptr<const UsesRelationship> uses_relationship = *iter;
-    sharedptr<const Relationship> relationship = uses_relationship->get_relationship();
-    if(relationship->get_has_fields()) //TODO: Handle related_record has_fields.
+    else
     {
-      sql_part_leftouterjoin += uses_relationship->get_sql_join_alias_definition();
-    }
-    else if(relationship->get_has_to_table())
-    {
-      //It is a relationship that only specifies the table, without specifying linking fields:
-      if(!(sql_part_from.empty()))
-        sql_part_from += ", ";
-
-      sql_part_from += relationship->get_to_table();
-    }
-  }
-
-  return sql_part_fields;
-}
-
-
-Glib::ustring Utils::build_sql_select_with_where_clause(const Glib::ustring& table_name, const type_vecConstLayoutFields& fieldsToGet, const Glib::ustring& where_clause, const Glib::ustring& extra_join, const type_sort_clause& sort_clause, const Glib::ustring& extra_group_by)
-{
-  //Get the list of fields to SELECT, plus the tables that they are selected FROM.
-  Glib::ustring sql_part_from;
-  Glib::ustring sql_part_leftouterjoin;
-  const Glib::ustring sql_part_fields = Utils::build_sql_select_fields_to_get(
-    table_name, fieldsToGet, sort_clause, sql_part_from, sql_part_leftouterjoin);
-
-  //Build the whole SQL statement:
-  Glib::ustring result = 
-    "SELECT " + sql_part_fields +
-    " FROM \"" + table_name + '\"';
-
-  if(!sql_part_from.empty())
-    result += (',' + sql_part_from);
-
-  if(!extra_join.empty())
-    sql_part_leftouterjoin += (' ' + extra_join + ' ');
-
-  if(!sql_part_leftouterjoin.empty())
-    result += (' ' + sql_part_leftouterjoin);
-
-
-  //Add the WHERE clause:
-  if(!where_clause.empty())
-    result += " WHERE " + where_clause;
-
-  //Extra GROUP_BY clause for doubly-related records. This must be before the ORDER BY sort clause:
-  if(!extra_group_by.empty())
-  {
-    result += (' ' + extra_group_by + ' ');
-  }
-
-  //Sort clause:
-  if(!sort_clause.empty())
-  {
-    Glib::ustring str_sort_clause;
-    for(type_sort_clause::const_iterator iter = sort_clause.begin(); iter != sort_clause.end(); ++iter)
-    {
-      sharedptr<const LayoutItem_Field> layout_item = iter->first;
-      if(layout_item)
+      const Glib::ustring field_name = layout_item->get_name();
+      if(!field_name.empty())
       {
-        const bool ascending = iter->second;
+        const Gnome::Gda::SqlBuilder::Id id = builder->select_add_field(field_name, parent);
 
-        if(!str_sort_clause.empty())
-          str_sort_clause += ", ";
-
-        str_sort_clause += "\"" + layout_item->get_sql_table_or_join_alias_name(table_name) + "\".\"" + layout_item->get_name() + "\" " + (ascending ? "ASC" : "DESC");
+        //Avoid duplicate records with doubly-related fields:
+        if(extra_join)
+          builder->select_group_by(id);
       }
     }
 
-    if(!str_sort_clause.empty())
-      result += " ORDER BY " + str_sort_clause;
+
+    one_added = true;
   }
 
-  return result;
+  if(!one_added)
+  {
+    std::cerr << G_STRFUNC << ": No fields added: fieldsToGet.size()=" << fieldsToGet.size() << std::endl;
+    return;
+  }
 }
 
 
-Glib::ustring Utils::build_sql_select_with_key(const Glib::ustring& table_name, const type_vecLayoutFields& fieldsToGet, const sharedptr<const Field>& key_field, const Gnome::Gda::Value& key_value)
+Glib::RefPtr<Gnome::Gda::SqlBuilder> Utils::build_sql_select_with_where_clause(const Glib::ustring& table_name, const type_vecConstLayoutFields& fieldsToGet, const Gnome::Gda::SqlExpr& where_clause, const sharedptr<const Relationship>& extra_join, const type_sort_clause& sort_clause, guint limit)
+{
+  Glib::RefPtr<Gnome::Gda::SqlBuilder> builder;
+
+  //Build the whole SQL statement:
+  try
+  {
+    builder = Gnome::Gda::SqlBuilder::create(Gnome::Gda::SQL_STATEMENT_SELECT);
+    builder->select_add_target(table_name);
+
+    //Add the fields to SELECT, plus the tables that they are selected FROM.
+    //We tell it whether extra_join is empty, so it can do an extra GROUP BY if necessary.
+    //TODO: Try to use DISTINCT instead, with a proper test case.
+    Utils::build_sql_select_add_fields_to_get(builder, table_name, fieldsToGet, sort_clause, extra_join /* bool */);
+
+    if(extra_join)
+    {
+      sharedptr<UsesRelationship> uses_relationship = sharedptr<UsesRelationship>::create();
+      uses_relationship->set_relationship(extra_join);
+      builder_add_join(builder, uses_relationship);
+    }
+
+    //Add the WHERE clause:
+    if(!where_clause.empty())
+    {
+      const int id = builder->import_expression(where_clause);
+      builder->set_where(id);
+    }
+
+    //Sort clause:
+    if(!sort_clause.empty())
+    {
+      for(type_sort_clause::const_iterator iter = sort_clause.begin(); iter != sort_clause.end(); ++iter)
+      {
+        sharedptr<const LayoutItem_Field> layout_item = iter->first;
+        if(layout_item)
+        {
+          const bool ascending = iter->second;
+
+          //TODO: Avoid the need for the "."
+          builder->select_order_by(
+            builder->add_field_id(layout_item->get_name(), layout_item->get_sql_table_or_join_alias_name(table_name)),
+            ascending);
+        }
+      }
+    }
+
+    //LIMIT clause:
+    if(limit > 0)
+    {
+      builder->select_set_limit(limit);
+    }
+  }
+  catch(const Glib::Error& ex)
+  {
+    std::cerr << G_STRFUNC << ": Exception: " << ex.what() << std::endl;
+  }
+
+  return builder;
+}
+
+
+Glib::RefPtr<Gnome::Gda::SqlBuilder> Utils::build_sql_select_with_key(const Glib::ustring& table_name, const type_vecLayoutFields& fieldsToGet, const sharedptr<const Field>& key_field, const Gnome::Gda::Value& key_value, const type_sort_clause& sort_clause, guint limit)
 {
   //TODO_Performance:
   type_vecConstLayoutFields constFieldsToGet;
@@ -399,108 +436,160 @@ Glib::ustring Utils::build_sql_select_with_key(const Glib::ustring& table_name,
     constFieldsToGet.push_back(*iter);
   }
 
-  return build_sql_select_with_key(table_name, constFieldsToGet, key_field, key_value);
-
-
+  return build_sql_select_with_key(table_name, constFieldsToGet, key_field, key_value, sort_clause, limit);
 }
 
-Glib::ustring Utils::build_sql_select_with_key(const Glib::ustring& table_name, const type_vecConstLayoutFields& fieldsToGet, const sharedptr<const Field>& key_field, const Gnome::Gda::Value& key_value)
+Gnome::Gda::SqlExpr Utils::build_simple_where_expression(const Glib::ustring& table_name, const sharedptr<const Field>& key_field, const Gnome::Gda::Value& key_value)
 {
-  if(!Conversions::value_is_empty(key_value)) //If there is a record to show:
+  Glib::RefPtr<Gnome::Gda::SqlBuilder> builder = Gnome::Gda::SqlBuilder::create(Gnome::Gda::SQL_STATEMENT_SELECT);
+  builder->select_add_target(table_name);  //This might not be necessary.
+  const Gnome::Gda::SqlBuilder::Id id = builder->add_cond(Gnome::Gda::SQL_OPERATOR_TYPE_EQ,
+    builder->add_field_id(key_field->get_name(), table_name),
+    builder->add_expr(key_value));
+  builder->set_where(id); //This might not be necessary.
+
+  return builder->export_expression(id);
+}
+
+Gnome::Gda::SqlExpr Utils::build_combined_where_expression(const Gnome::Gda::SqlExpr& a, const Gnome::Gda::SqlExpr& b, Gnome::Gda::SqlOperatorType op)
+{
+  Glib::RefPtr<Gnome::Gda::SqlBuilder> builder =
+    Gnome::Gda::SqlBuilder::create(Gnome::Gda::SQL_STATEMENT_SELECT);
+
+  const Gnome::Gda::SqlBuilder::Id id = builder->add_cond(op,
+    builder->import_expression(a),
+    builder->import_expression(b));
+   builder->set_where(id);
+  return builder->export_expression(id);
+}
+
+Glib::RefPtr<Gnome::Gda::SqlBuilder> Utils::build_sql_select_with_key(const Glib::ustring& table_name, const type_vecConstLayoutFields& fieldsToGet, const sharedptr<const Field>& key_field, const Gnome::Gda::Value& key_value, const type_sort_clause& sort_clause, guint limit)
+{
+  //We choose instead to have no where clause in this case,
+  //because that is useful to some callers:
+  //if(Conversions::value_is_empty(key_value)) //If there is a record to show:
+  //  return Glib::RefPtr<Gnome::Gda::SqlBuilder>();
+
+  Gnome::Gda::SqlExpr where_clause;
+  if(!Conversions::value_is_empty(key_value) && key_field)
   {
-    //TODO: Use a SQL parameter instead of using sql():
-    const Glib::ustring where_clause = '\"' + table_name + "\".\"" + key_field->get_name() + "\" = " + key_field->sql(key_value);
-    return Utils::build_sql_select_with_where_clause(table_name, fieldsToGet, where_clause);
+    where_clause = build_simple_where_expression(table_name, key_field, key_value);
   }
 
-  return Glib::ustring();
+  return Utils::build_sql_select_with_where_clause(table_name, fieldsToGet, where_clause,
+    sharedptr<const Relationship>(), sort_clause, limit);
 }
 
-Utils::type_list_values_with_second Utils::get_choice_values(const sharedptr<const LayoutItem_Field>& field)
+Utils::type_list_values_with_second Utils::get_choice_values_all(const Document* document, const sharedptr<const LayoutItem_Field>& field, sharedptr<const LayoutItem_Field>& layout_choice_first, sharedptr<const LayoutItem_Field>& layout_choice_second)
 {
-  type_list_values_with_second list_values;
+  return get_choice_values(document, field,
+    Gnome::Gda::Value() /* means get all with no WHERE clause */,
+    layout_choice_first, layout_choice_second);
+}
 
-  sharedptr<Relationship> choice_relationship;
+Utils::type_list_values_with_second Utils::get_choice_values(const Document* document, const sharedptr<const LayoutItem_Field>& field, const Gnome::Gda::Value& foreign_key_value, sharedptr<const LayoutItem_Field>& layout_choice_first, sharedptr<const LayoutItem_Field>& layout_choice_second)
+{
+  //Initialize output parameters:
+  layout_choice_first = sharedptr<LayoutItem_Field>();
+  layout_choice_second = sharedptr<LayoutItem_Field>();
+
+  //TODO: Reduce duplication between this and get_choice_values(field).
+
+  type_list_values_with_second result;
+
+  //We allows this, so this method can be used to get all records in a related table:
+  /*
+  if(Conversions::value_is_empty(foreign_key_value))
+  {
+    std::cout << G_STRFUNC << "debug: foreign_key_value is empty." << std::endl;
+    return result;
+  }
+  */
+
+  const FieldFormatting& format = field->get_formatting_used();
+  sharedptr<const Relationship> choice_relationship;
   Glib::ustring choice_field, choice_second;
-  field->get_formatting_used().get_choices(choice_relationship, choice_field, choice_second);
+  bool choice_show_all = false;
+  format.get_choices_related(document, choice_relationship, layout_choice_first, layout_choice_second, choice_show_all);
+
   if(!choice_relationship)
   {
-    //std::cout <<" debug: field has no choices: " << field->get_name() << std::endl;
-    return list_values;
+    std::cerr << G_STRFUNC << ": !choice_relationship." << std::endl;
+    return result;
   }
 
+  Utils::type_vecConstLayoutFields fields;
+  fields.push_back(layout_choice_first);
+  if(layout_choice_second)
+    fields.push_back(layout_choice_second);
+
   const Glib::ustring to_table = choice_relationship->get_to_table();
-  if(to_table.empty())
+  sharedptr<Field> to_field = document->get_field(to_table, choice_relationship->get_to_field());
+
+  if(!to_field)
   {
-    g_warning("get_choice_values(): table_name is null. relationship name = %s", glom_get_sharedptr_name(choice_relationship).c_str());
-    return list_values;
+    std::cerr << G_STRFUNC << ": to_field is null." << std::endl;
   }
 
-  const bool with_second = !choice_second.empty();
-  const Glib::ustring sql_second = "\"" + to_table + "\".\"" + choice_second + '\"';
+  type_sort_clause sort_clause;
+  sort_clause.push_back( type_pair_sort_field(layout_choice_first, true /* ascending */));
 
-  //Get possible values from database, sorted by the first column.
-  Glib::ustring sql_query = "SELECT \"" + to_table + "\".\"" + choice_field + '\"';
-  if(with_second)
-    sql_query += ", " + sql_second;
+  //TODO: Support related relationships (in the UI too):
+  Glib::RefPtr<Gnome::Gda::SqlBuilder> builder = Utils::build_sql_select_with_key(
+    to_table,
+    fields,
+    to_field,
+    foreign_key_value,
+    sort_clause);
 
-  sql_query += " FROM \"" + choice_relationship->get_to_table() + "\" ORDER BY \"" + to_table + "\".\"" + choice_field + '\"';
+  if(!builder)
+  {
+    std::cerr << G_STRFUNC << ": builder is null." << std::endl;
+    return result;
+  }
 
-  //std::cout << "debug: get_choice_values(): query: " << sql_query << std::endl;
-  //Connect to database:
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
+  //TODO: builder->select_order_by(choice_field_id);
+
+  //Connect to database and get the related values:
   sharedptr<SharedConnection> connection = ConnectionPool::get_instance()->connect();
-#else
-  std::auto_ptr<ExceptionConnection> conn_error;
-  sharedptr<SharedConnection> connection = ConnectionPool::get_instance()->connect(conn_error);
-  if(conn_error.get())
-    return list_values;
-#endif
 
   if(!connection)
-    return list_values;
+  {
+    std::cerr << G_STRFUNC << ": connection is null." << std::endl;
+    return result;
+  }
 
-  //std::cout << "get_choice_values: Executing SQL: " << sql_query << std::endl;
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
+  const std::string sql_query =
+    Utils::sqlbuilder_get_full_query(builder);
+  //std::cout << "debug: sql_query=" << sql_query << std::endl;
   Glib::RefPtr<Gnome::Gda::DataModel> datamodel = connection->get_gda_connection()->statement_execute_select(sql_query);
-#else
-  std::auto_ptr<Glib::Error> error;
-  Glib::RefPtr<Gnome::Gda::DataModel> datamodel = connection->get_gda_connection()->statement_execute_select(sql_query, Gnome::Gda::STATEMENT_MODEL_RANDOM_ACCESS, error);
-#endif
 
   if(datamodel)
   {
     const guint count = datamodel->get_n_rows();
-    //std::cout << "  result: count=" << count << std::endl;
+    const guint cols_count = datamodel->get_n_columns();
     for(guint row = 0; row < count; ++row)
     {
 
       std::pair<Gnome::Gda::Value, Gnome::Gda::Value> itempair;
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
       itempair.first = datamodel->get_value_at(0, row);
 
-      if(with_second)
+      if(layout_choice_second && (cols_count > 1))
         itempair.second = datamodel->get_value_at(1, row);
-#else
-      itempair.first = datamodel->get_value_at(0, row, error);
 
-      if(with_second)
-        itempair.second = datamodel->get_value_at(1, row, error);
-#endif
-
-      list_values.push_back(itempair);      
+      result.push_back(itempair);
     }
   }
   else
   {
-      std::cerr << "Glom  get_choice_values(): Error while executing SQL" << std::endl <<
+      std::cerr << G_STRFUNC << ": Error while executing SQL" << std::endl <<
                    "  " <<  sql_query << std::endl;
+      return result;
   }
 
-  return list_values;
+  return result;
 }
 
-
 Glib::ustring Utils::create_name_from_title(const Glib::ustring& title)
 {
   Glib::ustring result = string_replace(title, " ", "");
@@ -521,8 +610,8 @@ Glib::ustring Utils::string_escape_underscores(const Glib::ustring& text)
   return result;
 }
 
-/** Get just the first part of a locale, such as de_DE, 
- * ignoring, for instance, .UTF-8 or @euro at the end.
+/** Get just the first part of a locale, such as de_DE,
+ * ignoring, for instance, .UTF-8 or \@euro at the end.
  */
 Glib::ustring Utils::locale_simplify(const Glib::ustring& locale_id)
 {
@@ -566,7 +655,7 @@ Glib::ustring Utils::create_local_image_uri(const Gnome::Gda::Value& value)
 
   if(value.get_value_type() == GDA_TYPE_BINARY)
   {
-    glong size = 0;
+    long size = 0;
     gconstpointer pData = value.get_binary(size);
     if(size && pData)
     {
@@ -585,10 +674,10 @@ Glib::ustring Utils::create_local_image_uri(const Gnome::Gda::Value& value)
       }
     }
     else
-       std::cerr << "Utils::create_local_image_uri(): binary GdaValue contains no data." << std::endl;
+       std::cerr << G_STRFUNC << ": binary GdaValue contains no data." << std::endl;
   }
   //else
-  //  std::cerr << "Utils::create_local_image_uri(): type != BINARY" << std::endl;
+  //  std::cerr << G_STRFUNC << ": type != BINARY" << std::endl;
 
   if(result.empty())
     result = "/tmp/glom_report_image_invalid.png";
@@ -639,7 +728,7 @@ Glib::ustring Utils::title_from_string(const Glib::ustring& text)
 
 Utils::type_vec_strings Utils::string_separate(const Glib::ustring& str, const Glib::ustring& separator, bool ignore_quoted_separator)
 {
-  //std::cout << "Utils::string_separate(): separator=" << separator << std::endl;
+  //std::cout << "debug: " << G_STRFUNC << ": separator=" << separator << std::endl;
 
   type_vec_strings result;
 
@@ -647,13 +736,13 @@ Utils::type_vec_strings Utils::string_separate(const Glib::ustring& str, const G
   const Glib::ustring::size_type size_separator = separator.size();
 
   //A stack of quotes, so that we can handle nested quotes, whether they are " or ':
-  typedef std::stack<Glib::ustring> type_queue_quotes; 
+  typedef std::stack<Glib::ustring> type_queue_quotes;
   type_queue_quotes m_current_quotes;
 
   Glib::ustring::size_type unprocessed_start = 0;
   Glib::ustring::size_type item_start = 0;
   while(unprocessed_start < size)
-  { 
+  {
     //std::cout << "while unprocessed: un_processed_start=" << unprocessed_start << std::endl;
     Glib::ustring::size_type posComma = str.find(separator, unprocessed_start);
 
@@ -666,18 +755,18 @@ Utils::type_vec_strings Utils::string_separate(const Glib::ustring& str, const G
       if(ignore_quoted_separator)
       {
         //std::cout << "  debug: attempting to ignore quoted separators: " << separator << std::endl;
-       
+
         Glib::ustring::size_type posLastQuote = unprocessed_start;
 
         //std::cout << "    debug: posLastQuote=" << posLastQuote << std::endl;
         //std::cout << "    debug: posComma=" << posComma << std::endl;
- 
-  
+
+
         bool bContinue = true;
         while(bContinue && (posLastQuote < posComma))
         {
           //std::cout << "  continue" << std::endl;
-          Glib::ustring closing_quote; 
+          Glib::ustring closing_quote;
           if(!m_current_quotes.empty())
             closing_quote = m_current_quotes.top();
 
@@ -752,7 +841,7 @@ Utils::type_vec_strings Utils::string_separate(const Glib::ustring& str, const G
         // Do not add this item to the result, because it was quoted.
         continue;
       }
-     
+
       unprocessed_start = posComma + size_separator; //The while loops stops when this is empty.
     }
     else //if no separator found:
@@ -818,12 +907,14 @@ Glib::ustring Utils::string_remove_suffix(const Glib::ustring& str, const Glib::
 
 bool Utils::file_exists(const Glib::ustring& uri)
 {
+  if(uri.empty())
+     return false;
+
   //Check whether file exists already:
   {
     // Try to examine the input file.
     Glib::RefPtr<Gio::File> file = Gio::File::create_for_uri(uri);
 
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
     try
     {
       return file->query_exists();
@@ -832,11 +923,206 @@ bool Utils::file_exists(const Glib::ustring& uri)
     {
       return false; //Something went wrong. It does not exist.
     }
-#else
-      return file->query_exists();
-#endif
   }
 }
 
+std::string Utils::sqlbuilder_get_full_query(
+    const Glib::RefPtr<Gnome::Gda::Connection>& connection,
+    const Glib::ustring& query,
+    const Glib::RefPtr<const Gnome::Gda::Set>& params)
+{
+  Glib::ustring result = "glom_query_not_parsed";
+
+  try
+  {
+    Glib::RefPtr<Gnome::Gda::SqlParser> parser = connection->create_parser();
+    if(parser)
+    {
+      Glib::RefPtr<Gnome::Gda::Statement> stmt = parser->parse_string(query);
+      if(stmt)
+        result = stmt->to_sql(params);
+    }
+  }
+  catch(const Glib::Exception& ex)
+  {
+    std::cerr << G_STRFUNC << ": exception while parsing query: " << ex.what() << std::endl;
+  }
+  catch(const std::exception& ex)
+  {
+    std::cerr << G_STRFUNC << ": exception while parsing query: " << ex.what() << std::endl;
+  }
+
+  //Convert to something that std::cout should be able to handle.
+  const Glib::ScopedPtr<char> buf(g_convert_with_fallback(
+    result.raw().data(), result.raw().size(),
+    "ISO-8859-1", "UTF-8",
+    (char*)"?",
+    0, 0, 0));
+  return std::string(buf.get());
+}
+
+std::string Utils::sqlbuilder_get_full_query(
+  const Glib::RefPtr<const Gnome::Gda::SqlBuilder>& builder)
+{
+  Glib::ustring result = "glom_query_not_parsed";
+
+  try
+  {
+    Glib::RefPtr<Gnome::Gda::Statement> stmt = builder->get_statement();
+    if(stmt)
+      result = stmt->to_sql();
+  }
+  catch(const Gnome::Gda::SqlError& ex)
+  {
+    std::cerr << G_STRFUNC << ": SqlError exception while getting query: " << ex.what() << std::endl;
+  }
+  catch(const Glib::Exception& ex)
+  {
+    std::cerr << G_STRFUNC << ": exception (" << typeid(ex).name() << ") while getting query: " << ex.what() << std::endl;
+  }
+  catch(const std::exception& ex)
+  {
+    std::cerr << G_STRFUNC << ": exception (" << typeid(ex).name() << ") while getting query: " << ex.what() << std::endl;
+  }
+
+  //Convert to something that std::cout should be able to handle.
+  const Glib::ScopedPtr<char> buf(g_convert_with_fallback(
+    result.raw().data(), result.raw().size(),
+    "ISO-8859-1", "UTF-8",
+    (char*)"?",
+    0, 0, 0));
+  return std::string(buf.get());
+}
+
+Gnome::Gda::SqlExpr Utils::get_find_where_clause_quick(const Document* document, const Glib::ustring& table_name, const Gnome::Gda::Value& quick_search)
+{
+  Glib::RefPtr<Gnome::Gda::SqlBuilder> builder =
+    Gnome::Gda::SqlBuilder::create(Gnome::Gda::SQL_STATEMENT_SELECT);
+  builder->select_add_target(table_name);
+
+  if(!document)
+  {
+    std::cerr << G_STRFUNC << ": document was null." << std::endl;
+    return Gnome::Gda::SqlExpr();
+  }
+
+  //TODO: Cache the list of all fields, as well as caching (m_Fields) the list of all visible fields:
+  const Document::type_vec_fields fields = document->get_table_fields(table_name);
+
+  guint previous_and_id = 0;
+  typedef std::vector< sharedptr<LayoutItem_Field> > type_vecLayoutFields;
+  type_vecLayoutFields fieldsToGet;
+  for(Document::type_vec_fields::const_iterator iter = fields.begin(); iter != fields.end(); ++iter)
+  {
+    Glib::ustring strClausePart;
+
+    sharedptr<const Field> field = *iter;
+
+    bool use_this_field = true;
+    if(field->get_glom_type() != Field::TYPE_TEXT)
+    {
+      use_this_field = false;
+    }
+
+    if(use_this_field)
+    {
+      const guint eq_id = builder->add_cond(Gnome::Gda::SQL_OPERATOR_TYPE_EQ, //TODO: Ue field->sql_find_operator().
+        builder->add_field_id(field->get_name(), table_name),
+        builder->add_expr(quick_search)); //Use  field->sql_find(quick_search);
+
+      guint and_id = 0;
+      if(previous_and_id)
+      {
+        and_id = builder->add_cond(Gnome::Gda::SQL_OPERATOR_TYPE_AND,
+        previous_and_id, eq_id);
+      }
+
+      previous_and_id = and_id;
+    }
+  }
+
+  if(previous_and_id)
+  {
+    builder->set_where(previous_and_id); //This might be unnecessary.
+    return builder->export_expression(previous_and_id);
+  }
+  else
+  {
+    return Gnome::Gda::SqlExpr();
+  }
+}
+
+bool Utils::delete_directory(const Glib::RefPtr<Gio::File>& directory)
+{
+  if(!(directory->query_exists()))
+    return true;
+
+  //(Recursively) Delete any child files and directories,
+  //so we can delete this directory.
+  Glib::RefPtr<Gio::FileEnumerator> enumerator = directory->enumerate_children();
+
+  Glib::RefPtr<Gio::FileInfo> info = enumerator->next_file();
+  while(info)
+  {
+    Glib::RefPtr<Gio::File> child = directory->get_child(info->get_name());
+    bool removed_child = false;
+    if(child->query_file_type() == Gio::FILE_TYPE_DIRECTORY)
+      removed_child = delete_directory(child);
+    else
+      removed_child = child->remove();
+
+    if(!removed_child)
+       return false;
+
+    info = enumerator->next_file();
+  }
+
+  //Delete the actual directory:
+  if(!directory->remove())
+    return false;
+
+  return true;
+}
+
+bool Utils::delete_directory(const std::string& uri)
+{
+  Glib::RefPtr<Gio::File> file = Gio::File::create_for_uri(uri);
+  return delete_directory(file);
+}
+
+/** For instance, to find the first file in the directory with a .glom extension.
+ */
+Glib::ustring Utils::get_directory_child_with_suffix(const Glib::ustring& uri_directory, const std::string& suffix, bool recursive)
+{
+  Glib::RefPtr<Gio::File> directory = Gio::File::create_for_uri(uri_directory);
+  Glib::RefPtr<Gio::FileEnumerator> enumerator = directory->enumerate_children();
+
+  Glib::RefPtr<Gio::FileInfo> info = enumerator->next_file();
+  while(info)
+  {
+    Glib::RefPtr<const Gio::File> child = directory->get_child(info->get_name());
+
+    const Gio::FileType file_type = child->query_file_type();
+    if(file_type == Gio::FILE_TYPE_REGULAR)
+    {
+      //Check the filename:
+      const std::string basename = child->get_basename();
+      if(string_remove_suffix(basename, suffix) != basename)
+        return child->get_uri();
+    }
+    else if(recursive && file_type == Gio::FILE_TYPE_DIRECTORY)
+    {
+      //Look in sub-directories too:
+      const Glib::ustring result = get_directory_child_with_suffix(child->get_uri(), suffix, recursive);
+      if(!result.empty())
+        return result;
+    }
+
+    info = enumerator->next_file();
+  }
+
+  return Glib::ustring();
+}
+
 
 } //namespace Glom
