@@ -18,12 +18,13 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include "box_data_list_related.h"
-#include "dialog_layout_list_related.h"
-#include <glom/libglom/data_structure/glomconversions.h>
-#include <glom/libglom/glade_utils.h>
+#include <glom/mode_data/box_data_list_related.h>
+#include <glom/mode_design/layout/dialog_layout_list_related.h>
+#include <libglom/data_structure/glomconversions.h>
+#include <libglom/db_utils.h>
+#include <glom/glade_utils.h>
 #include <glom/frame_glom.h> //For show_ok_dialog()
-#include <bakery/App/App_Gtk.h> //For util_bold_message().
+#include <glom/utils_ui.h> //For bold_message()).
 #include <glibmm/i18n.h>
 
 namespace Glom
@@ -33,7 +34,6 @@ Box_Data_List_Related::Box_Data_List_Related()
 {
   set_size_request(400, -1); //An arbitrary default.
 
-  m_AddDel.set_rules_hint(); //Use alternating row colors when the theme does that.
   m_Alignment.add(m_AddDel);
   add_view(&m_AddDel); //Give it access to the document.
   m_AddDel.show();
@@ -57,8 +57,15 @@ Box_Data_List_Related::Box_Data_List_Related()
 
 void Box_Data_List_Related::enable_buttons()
 {
-  const bool view_details_possible = get_has_suitable_record_to_view_details();
-  m_AddDel.set_allow_view_details(view_details_possible); //Don't allow the user to go to a record in a hidden table.
+  const bool view_details_possible = 
+    get_has_suitable_record_to_view_details() && 
+    (m_portal->get_navigation_type() != LayoutItem_Portal::NAVIGATION_NONE);
+
+  #ifndef GLOM_ENABLE_MAEMO
+  // Don't allow the user to go to a record in a hidden table.
+  // Unless we are on Maemo - then we want to allow editing in a separate window only.
+  m_AddDel.set_allow_view_details(view_details_possible); 
+  #endif //GLOM_ENABLE_MAEMO
 }
 
 bool Box_Data_List_Related::init_db_details(const sharedptr<const LayoutItem_Portal>& portal, bool show_title)
@@ -89,7 +96,7 @@ bool Box_Data_List_Related::init_db_details(const Glib::ustring& parent_table, b
       relationship_title = _("Undefined Table");
     }
   
-    m_Label.set_markup(Bakery::App_Gtk::util_bold_message(relationship_title));
+    m_Label.set_markup(Utils::bold_message(relationship_title));
     m_Label.show();
 
     m_Alignment.set_padding(Utils::DEFAULT_SPACING_SMALL /* top */, 0, Utils::DEFAULT_SPACING_LARGE /* left */, 0);
@@ -202,7 +209,23 @@ void Box_Data_List_Related::on_adddel_script_button_clicked(const sharedptr<cons
     return;
   
   const Gnome::Gda::Value primary_key_value = get_primary_key_value(row);
-  execute_button_script(layout_item, primary_key_value);
+
+  // TODO: Calling refresh_data_from_database(), 
+  // or navigating to a different table from inside the Python script,
+  // causes a crash somewhere down in GTK+, so it is done in an idle handler here.
+  // We are currently in a callback from the CellRendererButton_Text cell
+  // renderer which is deleted by a call to refresh_data_from_database().
+  // Probably this causes issues somewhere.
+  Glib::signal_idle().connect(
+    sigc::bind(
+      sigc::mem_fun(*this, &Box_Data_List_Related::on_script_button_idle), 
+      layout_item,
+      primary_key_value));
+}
+
+bool Box_Data_List_Related::on_script_button_idle(const sharedptr<const LayoutItem_Button>& layout_item, const Gnome::Gda::Value& primary_key)
+{
+  execute_button_script(layout_item, primary_key);
 
   // Refill view from database as the script might have changed arbitrary records
 
@@ -215,19 +238,6 @@ void Box_Data_List_Related::on_adddel_script_button_clicked(const sharedptr<cons
     db_model->refresh_from_database(m_found_set);
 #endif
 
-  // TODO: Calling refresh_data_from_database() causes a crash somewhere
-  // down in GTK+, so it is done in a handler here.
-  // We are currently in a callback from the CellRendererButton_Text cell
-  // renderer which is deleted by a call to refresh_data_from_database().
-  // Probably this causes issues somewhere. 
-  Glib::signal_idle().connect(sigc::bind(sigc::mem_fun(*this, &Box_Data_List_Related::on_script_button_idle), primary_key_value));
-
-  //refresh_data_from_database();
-  //set_primary_key_value_selected(primary_key);
-}
-
-bool Box_Data_List_Related::on_script_button_idle(const Gnome::Gda::Value& primary_key)
-{
   refresh_data_from_database();
   set_primary_key_value_selected(primary_key);
   return false;
@@ -235,7 +245,8 @@ bool Box_Data_List_Related::on_script_button_idle(const Gnome::Gda::Value& prima
 
 void Box_Data_List_Related::on_adddel_record_added(const Gtk::TreeModel::iterator& row, const Gnome::Gda::Value& primary_key_value)
 {
-  //Note that on_record_added() would only be called on the AddDel itself, so we need ot handle this AddDel signal.
+  //Note that on_record_added() would only be called on the AddDel itself, 
+  //so we need to handle this AddDel signal.
   
   //primary_key_value is a new autogenerated or human-entered key for the row.
   //It has already been added to the database.
@@ -276,26 +287,15 @@ void Box_Data_List_Related::on_adddel_record_added(const Gtk::TreeModel::iterato
     //Create the link by setting the foreign key
     if(m_key_field && m_portal)
     {
-      Glib::RefPtr<Gnome::Gda::Set> params = Gnome::Gda::Set::create();
-      params->add_holder(m_key_field->get_holder(m_key_value));
-      params->add_holder(field_primary_key->get_holder(primary_key_value));
-      Glib::ustring strQuery = "UPDATE \"" + m_portal->get_table_used(Glib::ustring() /* not relevant */) + "\"";
-      strQuery += " SET \"" +  /* get_table_name() + "." +*/ m_key_field->get_name() + "\" = " + m_key_field->get_gda_holder_string();
-      strQuery += " WHERE \"" + get_table_name() + "\".\"" + field_primary_key->get_name() + "\" = " + field_primary_key->get_gda_holder_string();
-      //std::cout << "Box_Data_List_Related::on_adddel_record_added(): setting value in db=" << primary_key_value.to_string() << std::endl;
-      const bool test = query_execute(strQuery, params);
-      if(test)
-      {
-        //Show it on the view, if it's visible:
-        sharedptr<LayoutItem_Field> layout_item = sharedptr<LayoutItem_Field>::create();
-        layout_item->set_full_field_details(m_key_field);
+      make_record_related(primary_key_value);
+      
+      //Show it on the view, if it's visible:
+      sharedptr<LayoutItem_Field> layout_item = sharedptr<LayoutItem_Field>::create();
+      layout_item->set_full_field_details(m_key_field);
 
-        //TODO: Although the to-field value is visible on the new related record, get_value() returns NULL so you can't immediately navigate to the new record: 
-        //std::cout << "DEBUG: Box_Data_List_Related::on_record_added(): setting field=" << layout_item->get_name() << "m_key_value=" << m_key_value.to_string() << std::endl; 
-        m_AddDel.set_value(row, layout_item, m_key_value);
-      }
-      else
-        std::cerr << "Box_Data_List_Related::on_record_added(): SQL query failed: " << strQuery << std::endl;
+      //TODO: Although the to-field value is visible on the new related record, get_value() returns NULL so you can't immediately navigate to the new record: 
+      //std::cout << "DEBUG: Box_Data_List_Related::on_record_added(): setting field=" << layout_item->get_name() << "m_key_value=" << m_key_value.to_string() << std::endl; 
+      m_AddDel.set_value(row, layout_item, m_key_value);
     }
     else
       std::cerr << "Box_Data_List_Related::on_record_added(): m_key_field is NULL" << std::endl;
@@ -311,7 +311,7 @@ void Box_Data_List_Related::on_adddel_record_added(const Gtk::TreeModel::iterato
 void Box_Data_List_Related::on_dialog_layout_hide()
 {
   Dialog_Layout_List_Related* dialog_related = dynamic_cast<Dialog_Layout_List_Related*>(m_pDialogLayout);
-  g_assert(dialog_related != NULL);
+  g_assert(dialog_related);
   m_portal = dialog_related->get_portal_layout();
 
 
@@ -332,21 +332,15 @@ void Box_Data_List_Related::on_dialog_layout_hide()
 #ifndef GLOM_ENABLE_CLIENT_ONLY
 Dialog_Layout* Box_Data_List_Related::create_layout_dialog() const
 {
-  Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(Utils::get_glade_file_path("glom_developer.glade"), "window_data_layout");
-  if(refXml)
-  {
-    Dialog_Layout_List_Related* dialog = 0;
-    refXml->get_widget_derived("window_data_layout", dialog);
-    return dialog;
-  }
-  
-  return NULL;
+  Dialog_Layout_List_Related* dialog = 0;
+  Glom::Utils::get_glade_widget_derived_with_warning(dialog);
+  return dialog;
 }
 
 void Box_Data_List_Related::prepare_layout_dialog(Dialog_Layout* dialog)
 {
   Dialog_Layout_List_Related* related_dialog = dynamic_cast<Dialog_Layout_List_Related*>(dialog);
-  g_assert(related_dialog != NULL);
+  g_assert(related_dialog);
 
   related_dialog->set_document(m_layout_name, m_layout_platform, get_document(), m_portal, m_parent_table);
 }
@@ -368,9 +362,9 @@ void Box_Data_List_Related::set_primary_key_value(const Gtk::TreeModel::iterator
 }
 
 
-Document_Glom::type_list_layout_groups Box_Data_List_Related::create_layout_get_layout()
+Document::type_list_layout_groups Box_Data_List_Related::create_layout_get_layout()
 {
-  Document_Glom::type_list_layout_groups result;
+  Document::type_list_layout_groups result;
 
   //Do not use get_data_layout_groups(m_layout_name).
   //instead do this:
@@ -386,7 +380,7 @@ void Box_Data_List_Related::create_layout()
 {
   Box_Data::create_layout(); //Fills m_TableFields.
 
-  const Document_Glom* pDoc = dynamic_cast<const Document_Glom*>(get_document());
+  const Document* pDoc = dynamic_cast<const Document*>(get_document());
   if(pDoc)
   {
     //Field Names:
@@ -406,10 +400,10 @@ void Box_Data_List_Related::create_layout()
       m_AddDel.set_key_field(field_primary_key);
  
       //This map of layout groups will also contain the field information from the database:
-      Document_Glom::type_list_layout_groups layout_groups = create_layout_get_layout();
+      Document::type_list_layout_groups layout_groups = create_layout_get_layout();
       
       //int debug_count = 0;
-      for(Document_Glom::type_list_layout_groups::const_iterator iter = layout_groups.begin(); iter != layout_groups.end(); ++iter)
+      for(Document::type_list_layout_groups::const_iterator iter = layout_groups.begin(); iter != layout_groups.end(); ++iter)
       {
         //std::cout << "Box_Data_List::create_layout() group number=" << debug_count;
         //debug_count++;
@@ -472,7 +466,7 @@ void Box_Data_List_Related::create_layout_add_group(const sharedptr<LayoutGroup>
       {
         //Check that the field really exists, to avoid SQL errors.
         //This could probably only happen if we have failed to rename something everywhere, when the user has renamed something.
-        if(!get_field_exists_in_database(child_field->get_table_used(Base_DB_Table::m_table_name), child_field->get_name()))
+        if(!DbUtils::get_field_exists_in_database(child_field->get_table_used(Base_DB_Table::m_table_name), child_field->get_name()))
         {
           std::cerr << "debug: Box_Data_List_Related::create_layout_add_group(): Field does not exist in database: table_name=" << child_field->get_table_used(Base_DB_Table::m_table_name) << ", field_name=" << child_field->get_name() << std::endl;
           continue;
@@ -484,6 +478,4 @@ void Box_Data_List_Related::create_layout_add_group(const sharedptr<LayoutGroup>
   }
 }
 
-
-
 } //namespace Glom
